//------------------------------------------------------------------------------
//
// File:    DEdge.h
//
// Purpose: Directed edge class for graph library
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 1.0 30.04.2004
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#ifndef __APSGraph__DEdge_h__
#define __APSGraph__DEdge_h__ 1

#include "Edge.h"

namespace aps {

  namespace apsgraph {

template <class TEdge,class TVertex>
class DEdge : public Edge<TEdge,TVertex>
{
  template <class,class> friend class DVertex;

  private:

    TEdge   * nextFromDEdge;
    TEdge   * prevFromDEdge;

    TEdge   * nextToDEdge;
    TEdge   * prevToDEdge;

    TVertex * vertexFrom;
    TVertex * vertexTo;

  protected:

    DEdge( TVertex * aFromVertex, TVertex * aToVertex ) : Edge<TEdge,TVertex>(),
           nextFromDEdge( 0 ), prevFromDEdge( 0 ), nextToDEdge( 0 ), prevToDEdge( 0 ),
           vertexFrom( aFromVertex ), vertexTo( aToVertex ) {}

    virtual ~DEdge( void ) {}

  public:

    TEdge * getNextFromDEdge( void ) const
      { return( nextFromDEdge ); }

    TEdge * getPrevFromDEdge( void ) const
      { return( prevFromDEdge ); }

    TEdge * getNextToDEdge( void ) const
      { return( nextToDEdge ); }

    TEdge * getPrevToDEdge( void ) const
      { return( prevToDEdge ); }

    TVertex * getFromDVertex( void ) const
      { return( vertexFrom ); }

    TVertex * getToDVertex( void ) const
      { return( vertexTo ); }

    virtual int check( void ) const;    

    virtual std::ostream & Print( std::ostream & s, const int Mod ) const;
};

}}

#include "DEdge_impl.h"

#endif

//---------------------------- End of file ---------------------------
