//------------------------------------------------------------------------------
//
// File:    DEdge_impl.h
//
// Purpose: Directed edge implementation for graph library
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 1.0 30.04.2004
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#include "DVertex.h"

namespace aps {

  namespace apsgraph {

template <class TEdge,class TVertex>
int DEdge<TEdge,TVertex> :: check( void ) const
{
  TEdge * edge;
  int     retCode;

  retCode = 1;

  edge = vertexFrom->getFirstFromDEdge();

  while( edge ) {
    if( edge == this ) {
      retCode = 0;
      break;
    }

    edge = edge->getNextFromDEdge();
  }

  if( retCode ) {
    return( 20 );
  }

  retCode = 1;

  edge = vertexTo->getFirstToDEdge();

  while( edge ) {
    if( edge == this ) {
      retCode = 0;
      break;
    }

    edge = edge->getNextToDEdge();
  }

  if( retCode ) {
    return( 21 );
  }

  return( 0 );
}

template <class TEdge,class TVertex>
std::ostream & DEdge<TEdge,TVertex> :: Print( std::ostream & s, const int Mod ) const
{
  s << " Dir edge: " << this << std::endl;

  if( Mod > 0 ) {
    vertexFrom->Print( s, Mod - 1 );
    vertexTo->Print( s, Mod - 1 );
  }

  return( s );
}

}}

//---------------------------- End of file ---------------------------
