//------------------------------------------------------------------------------
//
// File:    DGraph.h
//
// Purpose: Directed graph class for graph library
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 1.0 30.04.2004
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#ifndef __APSGraph__DGraph_h__
#define __APSGraph__DGraph_h__ 1

#include <stack>

#include "Graph.h"

namespace aps {

  namespace apsgraph {

template <class TEdge,class TVertex>
class DGraph : public Graph<TEdge,TVertex>
{
  protected:

    DGraph( void ) : Graph<TEdge,TVertex>() {}

    virtual ~DGraph( void ) {}

    virtual void addEdge( TEdge * edge );

    virtual void addVertex( TVertex * vertex );

  private:

    int findEdgeInVertex( TEdge * edge ) const;

    int findEdgeInGraph( TEdge * edge ) const;

    int CheckEdgesInVertex( TVertex * vertex ) const;

    int CheckEdgesConsistency( void ) const;

    int getEdgesInNodesCount( void ) const;

    int recFindPath( TVertex * fromVertex, TVertex * toVertex ) const;

    bool FindCycle( TVertex * vertex, TVertex * sourceVertex,
                    std::stack<GraphElem*> & stack,
                    std::list<std::list<GraphElem*> > & Cycles );

    void Unmark( TVertex * vertex );

  public:

    virtual TEdge * findEdge( const TVertex * fromVertex, const TVertex * toVertex ) const;

    virtual void removeEdge( TEdge * edge );

    virtual void removeVertex( TVertex * vertex );

    virtual void deleteAllEdges( void );

    virtual int checkGraph( void ) const;

    virtual std::ostream & Print( std::ostream & s, const int Mod ) const;

    //----------- Algorithms ----------------

    int findPath( TVertex * fromVertex, TVertex * toVertex ) const;

    int FindAllCycles( std::list<std::list<GraphElem*> > & Cycles );
};

}}

#include "DGraph_impl.h"

#endif

//---------------------------- End of file ---------------------------
