//------------------------------------------------------------------------------
//
// File:    DVertex.h
//
// Purpose: Directed vertex class for graph library
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 1.0 30.04.2004
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#ifndef __APSGraph__DVertex_h__
#define __APSGraph__DVertex_h__ 1

#include "Vertex.h"

namespace aps {

  namespace apsgraph {

template <class TEdge,class TVertex>
class DVertex : public Vertex<TEdge,TVertex>
{
  template <class,class> friend class DGraph;

  private:

    TEdge   * firstFromDEdge;
    TEdge   * lastFromDEdge;
    TEdge   * firstToDEdge;
    TEdge   * lastToDEdge;

    virtual void addFromDEdge( TEdge * edge );

    virtual void addToDEdge( TEdge * edge );

    virtual void removeFromEdge( TEdge * edge );

    virtual void removeToEdge( TEdge * edge );

    virtual void clearAllEdges( void );

  protected:

    DVertex( void ) : Vertex<TEdge,TVertex>(), firstFromDEdge( 0 ), lastFromDEdge( 0 ),
                      firstToDEdge( 0 ), lastToDEdge( 0 ) {}

    virtual ~DVertex( void ) {}

  public:

    TEdge * getFirstFromDEdge( void ) const
      { return( firstFromDEdge ); }

    TEdge * getLastFromDEdge( void ) const
      { return( lastFromDEdge ); }

    TEdge * getFirstToDEdge( void ) const
      { return( firstToDEdge ); }

    TEdge * getLastToDEdge( void ) const
      { return( lastToDEdge ); }

    int getFromEdgesCount( void ) const;

    int getToEdgesCount( void ) const;

    int getEdgesCount( void ) const;

    int IfEdgeInVertex( TEdge * edge ) const;

    virtual int check( void ) const;

    virtual std::ostream & Print( std::ostream & s, const int Mod ) const;
};

}}

#include "DVertex_impl.h"

#endif

//---------------------------- End of file ---------------------------
