//------------------------------------------------------------------------------
//
// File:    Graph.h
//
// Purpose: Graph class for graph library
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 1.0 30.04.2004
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#ifndef __APSGraph__Graph_h__
#define __APSGraph__Graph_h__ 1

#include "Base.h"

#include <string>

namespace aps {

  namespace apsgraph {

template <class TEdge,class TVertex>
class Graph : public Base
{
  private:

    TEdge   * firstEdge;
    TEdge   * lastEdge;
    TVertex * firstVertex;
    TVertex * lastVertex;    

  protected:

    Graph( void ) : Base(), firstEdge( 0 ), lastEdge( 0 ),
                    firstVertex( 0 ), lastVertex( 0 ) {};

    virtual ~Graph( void );

    virtual void addEdge( TEdge * edge );

    virtual void addVertex( TVertex * vertex );

  public:

    TEdge * getFirstEdge( void ) const
      { return( firstEdge ); }

    TEdge * getLastEdge( void ) const
      { return( lastEdge ); }

    TVertex * getFirstVertex( void ) const
      { return( firstVertex ); }

    TVertex * getLastVertex( void ) const
      { return( lastVertex ); }

    int getEdgesCount( void ) const;

    int getVerticesCount( void ) const;

    virtual TEdge * findEdge( const TVertex * fromVertex, const TVertex * toVertex ) const = 0;

    virtual void removeEdge( TEdge * edge ); 

    virtual int removeEdgeVertex( TVertex * fromVertex, TVertex * toVertex );

    virtual void removeVertex( TVertex * vertex );

    virtual void clearGraph( void );

    virtual void deleteAllEdges( void );

    void clearAllEdgesTmpFlags( void ) const;

    void clearAllVerticesTmpFlags( void ) const;

    void clearAllTmpFlags( void ) const;

    void clearAllVisitFlags( void ) const;

    virtual int checkGraph( void ) const;

    virtual void graphError( const int Mod, const std::string & Msg ) const;

    virtual std::ostream & Print( std::ostream & s, const int Mod ) const;
};

}}

#include "Graph_impl.h"

#endif

//---------------------------- End of file ---------------------------
