//------------------------------------------------------------------------------
//
// File:    UEdge.h
//
// Purpose: Undirected edge class for graph library
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 1.0 30.04.2004
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#ifndef __APSGraph__UEdge_h__
#define __APSGraph__UEdge_h__ 1

#include "Edge.h"

namespace aps {

  namespace apsgraph {

template <class,class> class UVertex;

template <class TEdge,class TVertex>
class UEdge : public Edge<TEdge,TVertex>
{
  template <class,class> friend class UVertex;

  private:

    TEdge   * nextUEdge1;
    TEdge   * prevUEdge1;

    TEdge   * nextUEdge2;
    TEdge   * prevUEdge2;

    TVertex * vertex1;
    TVertex * vertex2;

  protected:

    UEdge( TVertex * aVertex1, TVertex * aVertex2 ) : Edge<TEdge,TVertex>(),
           nextUEdge1( 0 ), prevUEdge1( 0 ), nextUEdge2( 0 ), prevUEdge2( 0 ),
           vertex1( aVertex1 ), vertex2( aVertex2 ) {}

    virtual ~UEdge( void ) {}

  public:

    TEdge * getNextUEdge( const UVertex<TEdge,TVertex> * aVertex ) const;

    TEdge * getPrevUEdge( const UVertex<TEdge,TVertex> * aVertex ) const;

    TVertex * getUVertex1( void ) const
      { return( vertex1 ); }

    TVertex * getUVertex2( void ) const
      { return( vertex2 ); }

    TVertex * GetAnotherVertex( const TVertex * vertex ) const;

    void swapVerticies( void );

    virtual int check( void ) const;    

    virtual std::ostream & Print( std::ostream & s, const int Mod ) const;
};

}}

#include "UEdge_impl.h"

#endif

//---------------------------- End of file ---------------------------
