//------------------------------------------------------------------------------
//
// File:    UEdge_impl.h
//
// Purpose: Undirected edge implementation for graph library
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 1.0 30.04.2004
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#include "UVertex.h"

namespace aps {

  namespace apsgraph {

template <class TEdge,class TVertex>
TEdge * UEdge<TEdge,TVertex> :: getNextUEdge( const UVertex<TEdge,TVertex> * aVertex ) const
{
  if( vertex1 == aVertex ) {
    return( nextUEdge1 );
  }

  if( vertex2 == aVertex ) {
    return( nextUEdge2 );
  }

  return( 0 );
}

template <class TEdge,class TVertex>
TEdge * UEdge<TEdge,TVertex> :: getPrevUEdge( const UVertex<TEdge,TVertex> * aVertex ) const
{
  if( vertex1 == aVertex ) {
    return( prevUEdge1 );
  }

  if( vertex2 == aVertex ) {
    return( prevUEdge2 );
  }

  return( 0 );
}

template <class TEdge,class TVertex>
TVertex * UEdge<TEdge,TVertex> :: GetAnotherVertex( const TVertex * vertex ) const
{
  if( vertex1 == vertex ) {
    return( vertex2 );
  }

  if( vertex2 == vertex ) {
    return( vertex1 );
  }

  return( 0 );
}

template <class TEdge,class TVertex>
void UEdge<TEdge,TVertex> :: swapVerticies( void )
{
  TEdge   * edge;
  TVertex * vertex;

  vertex = vertex1;
  vertex1 = vertex2;
  vertex2 = vertex;

  edge = nextUEdge1;
  nextUEdge1 = nextUEdge2;
  nextUEdge2 = edge;

  edge = prevUEdge1;
  prevUEdge1 = prevUEdge2;
  prevUEdge2 = edge;
}

template <class TEdge,class TVertex>
int UEdge<TEdge,TVertex> :: check( void ) const
{
  TEdge * edge;
  int     retCode;

  retCode = 1;

  edge = vertex1->getFirstUEdge();

  while( edge ) {
    if( edge == this ) {
      retCode = 0;
      break;
    }

    edge = edge->getNextUEdge( vertex1 );
  }

  if( retCode ) {
    return( 20 );
  }

  retCode = 1;

  edge = vertex2->getFirstUEdge();

  while( edge ) {
    if( edge == this ) {
      retCode = 0;
      break;
    }

    edge = edge->getNextUEdge( vertex2 );
  }

  if( retCode ) {
    return( 21 );
  }

  return( 0 );
}

template <class TEdge,class TVertex>
std::ostream & UEdge<TEdge,TVertex> :: Print( std::ostream & s, const int Mod ) const
{
  s << " Undir edge: " << this << std::endl;

  if( Mod > 0 ) {
    vertex1->Print( s, Mod - 1 );
    vertex2->Print( s, Mod - 1 );
  }

  return( s );
}

}}

//---------------------------- End of file ---------------------------
