//------------------------------------------------------------------------------
//
// File:    UGraph.h
//
// Purpose: Undirected graph class for graph library
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 1.0 30.04.2004
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#ifndef __APSGraph__UGraph_h__
#define __APSGraph__UGraph_h__ 1

#include "Graph.h"
#include <list>
#include <stack>

namespace aps {

  namespace apsgraph {

template <class TEdge,class TVertex>
class UGraph : public Graph<TEdge,TVertex>
{
  protected:

    UGraph( void ) : Graph<TEdge,TVertex>() {}

    virtual ~UGraph( void ) {}

    virtual void addEdge( TEdge * edge );

    virtual void addVertex( TVertex * vertex );

  private:

    int findEdgeInVertex( TEdge * edge ) const;

    int findEdgeInGraph( TEdge * edge ) const;

    int CheckEdgesInVertex( TVertex * vertex ) const;

    int CheckEdgesConsistency( void ) const;

    int getEdgesInNodesCount( void ) const;

    bool recFindPath( TVertex * fromVertex, TVertex * toVertex ) const;

    bool recFindHamiltonPath( TVertex * fromVertex, TVertex * toVertex, int Vertices,
                              std::list<GraphElem*> & coll ) const;

    bool recFindHamiltonCycle( TVertex * fromVertex, TVertex * toVertex, TEdge * FromEdge, const int Vertices,
                               std::list<GraphElem*> & coll ) const;

    int recRankingGraph( TVertex * fromVertex, TEdge * fromEdge, const int rank ) const;

    void recFindComponents( TVertex * fromVertex, const int component ) const;

    TEdge * getNextExistingEdge( TVertex * vertex ) const;

    TVertex * recFindEulerPath( TVertex * vertex, std::stack<TVertex*> & VStack ) const;

  public:

    virtual TEdge * findEdge( const TVertex * fromVertex, const TVertex * toVertex ) const;

    virtual void removeEdge( TEdge * edge );

    virtual void removeVertex( TVertex * vertex );

    virtual void deleteAllEdges( void );

    virtual int checkGraph( void ) const;

    virtual std::ostream & Print( std::ostream & s, const int Mod ) const;

    //----------- Algorithms ----------------

    bool findPath( TVertex * fromVertex, TVertex * toVertex ) const;

    bool findHamiltonPath( TVertex * fromVertex, TVertex * toVertex, std::list<GraphElem*> & coll ) const;

    bool findHamiltonCycle( std::list<GraphElem*> & coll ) const;

    int rankingGraph( TVertex * fromVertex ) const;

    int findComponents( void ) const;

    bool ifEulerPathExists( TVertex * fromVertex, TVertex * toVertex ) const;

    bool findEulerPath( TVertex * fromVertex, TVertex * toVertex, std::list<TVertex*> & coll ) const;
};

}}

#include "UGraph_impl.h"

#endif

//---------------------------- End of file ---------------------------
