//------------------------------------------------------------------------------
//
// File:    UVertex_impl.h
//
// Purpose: Undirected vertex implementation for graph library
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 1.0 30.04.2004
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#include "UEdge.h"

namespace aps {

  namespace apsgraph {

template <class TEdge,class TVertex>
void UVertex<TEdge,TVertex> :: addEdge1( TEdge * edge )
{
  if( lastEdge ) {
    edge->nextUEdge1     = 0;
    edge->prevUEdge1     = lastEdge;

    if( lastEdge->getUVertex1() == this ) {
      lastEdge->nextUEdge1 = edge;
    }

    if( lastEdge->getUVertex2() == this ) {
      lastEdge->nextUEdge2 = edge;
    }

    lastEdge             = edge;
  }
  else {
    edge->nextUEdge1 = 0;
    edge->prevUEdge1 = 0;
    firstEdge        = edge;
    lastEdge         = edge;
  }
}

template <class TEdge,class TVertex>
void UVertex<TEdge,TVertex> :: addEdge2( TEdge * edge )
{
  if( lastEdge ) {
    edge->nextUEdge2     = 0;
    edge->prevUEdge2     = lastEdge;

    if( lastEdge->getUVertex1() == this ) {
      lastEdge->nextUEdge1 = edge;
    }

    if( lastEdge->getUVertex2() == this ) {
      lastEdge->nextUEdge2 = edge;
    }

    lastEdge             = edge;
  }
  else {
    edge->nextUEdge2 = 0;
    edge->prevUEdge2 = 0;
    firstEdge        = edge;
    lastEdge         = edge;
  }
}

template <class TEdge,class TVertex>
void UVertex<TEdge,TVertex> :: removeEdge( TEdge * edge )
{
  TEdge * edgePrev = edge->getPrevUEdge( this );
  TEdge * edgeNext = edge->getNextUEdge( this );

  if( edgePrev ) {
    if( edgePrev->getUVertex1() == this ) {
      edgePrev->nextUEdge1 = edgeNext;
    }
    else {
      if( edgePrev->getUVertex2() == this ) {
        edgePrev->nextUEdge2 = edgeNext;
      }
    }
  }
  else {
    firstEdge = edgeNext;
  }

  if( edgeNext ) {
    if( edgeNext->getUVertex1() == this ) {
      edgeNext->prevUEdge1 = edgePrev;
    }
    else {
      if( edgeNext->getUVertex2() == this ) {
        edgeNext->prevUEdge2 = edgePrev;
      }
    }
  }
  else {
    lastEdge = edgePrev;
  }
}

template <class TEdge,class TVertex>
void UVertex<TEdge,TVertex> :: clearAllEdges( void )
{
  firstEdge = 0;
  lastEdge  = 0;
}

template <class TEdge,class TVertex>
int UVertex<TEdge,TVertex> :: getEdgesCount( void ) const
{
  int     count = 0;
  TEdge * edge  = firstEdge;

  while( edge ) {
    count++;

    edge = edge->getNextUEdge( this );
  }

  return( count );
}

template <class TEdge,class TVertex>
int UVertex<TEdge,TVertex> :: IfEdgeInVertex( TEdge * edge ) const
{
  TEdge * edge1  = firstEdge;

  while( edge1 ) {
    if( edge1 == edge ) {
      return( 1 );
    }

    edge1 = edge1->getNextUEdge( this );
  }

  return( 0 );
}

template <class TEdge,class TVertex>
int UVertex<TEdge,TVertex> :: check( void ) const
{
  TEdge   * edge;
  TEdge   * edgePrev;
  TEdge   * edgeNext;

  if( ( firstEdge && !lastEdge ) || ( !firstEdge && lastEdge ) ) {
    return( 1 );
  }

  edge = firstEdge;

  if( edge ) {
    if( edge->getPrevUEdge( this ) ) {
      return( 2 );
    }

    edgePrev = edge;
    edge     = edge->getNextUEdge( this ); 

    while( edge ) {
      if( edge->getPrevUEdge( this ) != edgePrev ) {
        return( 3 );
      }

      edgePrev = edge;
      edge = edge->getNextUEdge( this );
    }
  }

  edge = lastEdge;

  if( edge ) {
    if( edge->getNextUEdge( this ) ) {
      return( 4 );
    }

    edgeNext = edge;
    edge     = edge->getPrevUEdge( this ); 

    while( edge ) {
      if( edge->getNextUEdge( this ) != edgeNext ) {
        return( 5 );
      }

      edgeNext = edge;
      edge = edge->getPrevUEdge( this );
    }
  }

  return( 0 );
}

template <class TEdge,class TVertex>
std::ostream & UVertex<TEdge,TVertex> :: Print( std::ostream & s, const int Mod ) const
{
  TEdge * edge;

  s << " Undir vertex: " << this << " Visited = " << Vertex<TEdge,TVertex>::getVisit()
    << " Tmp = " << GraphElem::getTmpFlag()
    << " Edges = " << getEdgesCount() << std::endl;

  if( Mod > 0 ) {
    edge = firstEdge;

    while( edge ) {
      edge->Print( s, Mod - 1 );

      edge = edge->getNextUEdge( this );
    }
  }

  return( s );
}

}}

//---------------------------- End of file ---------------------------



